#!/bin/bash

SOURCE_DIR=$(realpath $(dirname "${0}"))

# Install dependencies for Sahana
apt-get -y --no-install-recommends install python-dateutil python-imaging python-lxml python-matplotlib python-openid python-psycopg2 python-reportlab python-requests python-shapely python-xlrd python-xlwt

# Install web2py
git clone --recursive https://github.com/web2py/web2py.git /srv/sahana
git -C /srv/sahana checkout cda35fd
git -C /srv/sahana submodule update

# Install Sahana
git clone https://github.com/sahana/eden.git /srv/sahana/applications/eden

# Create PostgreSQL user and database
export SAHANA_PWD=$(head -c 260 /dev/urandom | tr -cd '[:alnum:]' | head -c 26)
envsubst <${SOURCE_DIR}/sahana/tmp/sahana-createdb.sql >/tmp/sahana-createdb.sql
sudo -u postgres psql -f /tmp/sahana-createdb.sql
rm -f /tmp/sahana-createdb.sql

# Configure web2py
cp ${SOURCE_DIR}/sahana/srv/sahana/routes.py /srv/sahana/routes.py
cp ${SOURCE_DIR}/sahana/srv/sahana/run_scheduler.py /srv/sahana/run_scheduler.py
ln -s /srv/sahana/handlers/wsgihandler.py /srv/sahana/wsgihandler.py

# Configure Sahana
mkdir /srv/sahana/applications/eden/{databases,errors,sessions}
mkdir -p /srv/sahana/applications/eden/static/cache/chart
mkdir -p /srv/sahana/applications/eden/uploads/{gis_cache,images,tracks}
export SAHANA_HMAC=$(head -c 260 /dev/urandom | tr -cd '[:alnum:]' | head -c 26)
envsubst <${SOURCE_DIR}/sahana/srv/sahana/applications/eden/models/000_config.py >/srv/sahana/applications/eden/models/000_config.py
#cp ${SOURCE_DIR}/sahana/srv/sahana/applications/eden/modules/templates/default/config.py /srv/sahana/applications/eden/modules/templates/default/config.py
cp ${SOURCE_DIR}/sahana/srv/sahana/applications/eden/languages/cz.py /srv/sahana/applications/eden/languages/cz.py

# Patch bugs
#cp ${SOURCE_DIR}/sahana/srv/sahana/applications/eden/modules/s3/s3resource.py /srv/sahana/applications/eden/modules/s3/s3resource.py
#cp ${SOURCE_DIR}/sahana/srv/sahana/applications/eden/modules/s3db/delphi.py /srv/sahana/applications/eden/modules/s3db/delphi.py
#cp ${SOURCE_DIR}/sahana/srv/sahana/applications/eden/modules/s3db/org.py /srv/sahana/applications/eden/modules/s3db/org.py
#cp ${SOURCE_DIR}/sahana/srv/sahana/gluon/contrib/login_methods/openid_auth.py /srv/sahana/gluon/contrib/login_methods/openid_auth.py

# Create OS user
adduser --system --group --home /srv/sahana --shell /bin/false sahana
chown -R sahana:sahana /srv/sahana

# Populate database
cd /srv/sahana
sudo -u sahana python web2py.py -S eden -M -R applications/eden/static/scripts/tools/noop.py

# Set "production values" (increases performance)
sed -i 's/settings.base.migrate = True/settings.base.migrate = False/' /srv/sahana/applications/eden/models/000_config.py
sed -i 's/#settings.base.prepopulate = 0/settings.base.prepopulate = 0/' /srv/sahana/applications/eden/models/000_config.py
sudo -u sahana python web2py.py -S eden -M -R applications/eden/static/scripts/tools/compile.py

# Create admin account
export SAHANA_ADMIN_HASH=$(python ${SOURCE_DIR}/sahana/sahana-password-hash.py)
envsubst <${SOURCE_DIR}/sahana/tmp/sahana-adminaccount.sql >/tmp/sahana-adminaccount.sql
sudo -u postgres psql -f /tmp/sahana-adminaccount.sql sahana
rm /tmp/sahana-adminaccount.sql

# Create uwsgi and nginx app definition
cp ${SOURCE_DIR}/sahana/etc/uwsgi/apps-available/sahana.ini /etc/uwsgi/apps-available/sahana.ini
ln -s /etc/uwsgi/apps-available/sahana.ini /etc/uwsgi/apps-enabled/sahana.ini
cp ${SOURCE_DIR}/sahana/etc/nginx/apps-available/sahana /etc/nginx/apps-available/sahana
ln -s /etc/nginx/apps-available/sahana /etc/nginx/apps-enabled/sahana

# Restart services
systemctl restart uwsgi
systemctl restart nginx

# TODO: Set cgroups

# TODO: Does matplotlib still require MPLCONFIGDIR directory?
# If so, /srv/sahana/wsgihandler.py needs to be modified

# TODO: Potentially needed dependencies
# WARNING:  Setup unresolved dependency: boto required for Setup Module
# WARNING:  S3Doc unresolved dependency: pyth required for RTF document support in Full-Text Search
# WARNING:  S3Doc unresolved dependency: sunburnt required for Full-Text Search support
# WARNING:  S3GIS unresolved dependency: selenium required for Map printing support
# WARNING:  Setup unresolved dependency: PyYAML required for Setup Module
# WARNING:  S3GIS unresolved dependency: GDAL required for Shapefile support
# WARNING:  Setup unresolved dependency: ansible required for Setup Module
# WARNING:  Survey unresolved dependency: PyRTF required if you want to export assessment/survey templates as a Word document
# WARNING: Unresolved optional dependency: pyserial required
# WARNING:  S3Msg unresolved dependency tweepy required for non-Tropo Twitter support
# WARNING:  Message Parsing unresolved dependency: TwitterSearch required for fetching results from twitter keyword queries
